Type: Package
Package: apache.sedona
Title: R Interface for Apache Sedona
Version: 1.9.0
Authors@R: 
    c(person(family = "Apache Sedona",
             role = c("aut", "cre"),
             email = "private@sedona.apache.org"),
      person(given = "Jia",
             family = "Yu",
             role = c("ctb", "cph"),
             email = "jiayu@apache.org"),
      person(given = "Yitao",
             family = "Li",
             role = c("aut", "cph"),
             email = "yitao@rstudio.com",
             comment = c(ORCID = "0000-0002-1261-905X")),
      person(family = "The Apache Software Foundation",
             role = "cph"),
      person(family = "RStudio",
             role = "cph"))
Maintainer: Apache Sedona <private@sedona.apache.org>
Description: R interface for 'Apache Sedona' based on 'sparklyr'
    (<https://sedona.apache.org>).
License: Apache License 2.0
URL: https://github.com/apache/sedona/, https://sedona.apache.org/
BugReports: https://github.com/apache/sedona/issues
Depends: R (>= 3.2)
Imports: rlang, sparklyr (>= 1.3), dbplyr (>= 1.1.0), cli, lifecycle
Suggests: dplyr (>= 0.7.2), knitr, rmarkdown
Encoding: UTF-8
RoxygenNote: 7.3.2
SystemRequirements: 'Apache Spark' 3.x
NeedsCompilation: no
Packaged: 2026-04-23 06:36:58 UTC; jiayu
Author: Apache Sedona [aut, cre],
  Jia Yu [ctb, cph],
  Yitao Li [aut, cph] (ORCID: <https://orcid.org/0000-0002-1261-905X>),
  The Apache Software Foundation [cph],
  RStudio [cph]
Repository: CRAN
Date/Publication: 2026-04-23 07:30:03 UTC
