# Generated by roxygen2: do not edit by hand

S3method(sdf_register,spatial_rdd)
export(approx_count)
export(as.spark.dataframe)
export(crs_transform)
export(minimum_bounding_box)
export(new_bounding_box)
export(sedona_apply_spatial_partitioner)
export(sedona_build_index)
export(sedona_knn_query)
export(sedona_range_query)
export(sedona_read_dsv_to_typed_rdd)
export(sedona_read_geojson)
export(sedona_read_geojson_to_typed_rdd)
export(sedona_read_shapefile)
export(sedona_read_shapefile_to_typed_rdd)
export(sedona_read_wkb)
export(sedona_read_wkt)
export(sedona_render_choropleth_map)
export(sedona_render_heatmap)
export(sedona_render_scatter_plot)
export(sedona_save_spatial_rdd)
export(sedona_spatial_join)
export(sedona_spatial_join_count_by_key)
export(sedona_write_geojson)
export(sedona_write_wkb)
export(sedona_write_wkt)
export(spark_read_geojson)
export(spark_read_geoparquet)
export(spark_read_shapefile)
export(spark_write_geojson)
export(spark_write_geoparquet)
export(spark_write_raster)
export(to_spatial_rdd)
import(sparklyr)
importFrom(cli,cli_alert_info)
importFrom(lifecycle,deprecated)
importFrom(rlang,"%||%")
importFrom(sparklyr,sdf_register)
importFrom(sparklyr,spark_read_source)
importFrom(sparklyr,spark_write_source)
